//
//  world.h
//  Infektsimulation
//
//  Created by //// on 21.08.13.
//  Copyright (c) 2013 Alexander Droste & Florian Flachsenberg. All rights reserved.
//

#include <limits.h>
#include "host.h"
#include "agent.h"
#include <stdbool.h>
#include "datatypes.h"

#ifndef Infektsimulation_world_h
#define Infektsimulation_world_h

//radii of infection paths: normal, squared
#define radius_air 5
#define radius_skin 3
#define radius_oral 2
#define radius_xxx 1

typedef struct{
    Size size; //complete size of the world
    FieldMatrix* field_matrix; //matrix containing all fields
    Size size_of_one_field; //size of one field
    unsigned infected_total, immune_total, died_total, ill_total, ill_current, infected_current, healthy_current, infectious_dead_current;
    unsigned min_incubation, max_incubation, min_illness, max_illness;
    double mean_incubation, mean_illness;
    unsigned radius_air_sq, radius_oral_sq, radius_skin_sq, radius_xxx_sq, maxradius_sq;
}World;

Coordinates world_get_field_coord_for_host(Host* inc_host);
Field* world_get_field_for_host(Host* inc_host);
void world_transfer_host_to_field(Host* inc_host, Field* dest_field);
Position world_get_pos_within_field(unsigned int x, unsigned int y);
void world_add_host_to_field(Host* inc_host);
void world_remove_host_from_field(Host* host);
bool world_host_position_collides(Host* inc_host);
void world_create(unsigned int world_axes_length, unsigned int field_axes_length);
void world_create_for_agent(unsigned world_axes_length, Agent* agent);
bool world_host_position_collides(Host* inc_host);
#ifdef DRAW 
void world_draw(const char* prefix, unsigned suffix);
#endif
void world_simulate(unsigned short time_steps, unsigned stop_if_finished);

#endif
